#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 07/11/2002
################################################################################
#
# Board - Messages board functionality for Ethereal-Realms
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204, 817 5TH ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Text::Wrap;							# Text wrapping functionality
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Auditing handler
use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date processing
use Ethereal::Filter;						# Filter processing
use Ethereal::Login;						# Login functionality
use Ethereal::Mail;						# Mail handler
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table creation
use Ethereal::Template;						# Template handler


################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $tmpl;							# Template

my $ban;							# Banned status
my $script;							# Script name

my %board;							# Board hash
my %system;							# System hash

################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Retreive hash
	$database->GetHashBoard(\%board);
	$database->GetHashSystem(\%system);

	# Login handle
	$login    = new Ethereal::Login($database, $cgi, $param);
	$tmpl     = new Ethereal::Template(\%board);

	# Script name
	$script   = $cgi->url(-relative=>1);


	# System boards
	$param->{'ROOM'} = $system{'SetBoardAlias'} if ($script =~ /^sysboard/i);


	# Handle headers
	HandleAlternate($database, $cgi, $param);

	# Document header
	$database->DocumentGetHeader();

	if (RealmSelect($database, $cgi, $param, $tmpl, \%board))
	{
		# Proper default
		$param->{'TOPIC'} = ($param->{'TOPIC'} eq $board{'TxtDefault'}) ? 'system' : $param->{'TOPIC'};

		# Logic handling
		# Writer
		PostWrite($database, $login, $cgi, $param, \%board, \%system) if (defined($param->{'WRITE'})); 

		# Removal
		PostPurge($database, $login, $cgi, $param) if (defined($param->{'PURGE'}));


		# Composer
		if ((defined($param->{'REPLY'})) || (defined($param->{'POST'})))
		{
			# Composition
 			PostComposer($database, $login, $cgi, $param, $tmpl, \%board); 
		}

		# Posts viewer (default)
		elsif (!defined($param->{'PASSGO'}))
		{ 
			# Adjust title
			# Standard behaviour
			if ($script !~ /^sysboard/i)
			{ $tmpl->ShowMain($param->{'ROOM'}); }

			# System board
			else
			{ $tmpl->ShowMain($system{'SetBoardName'}); }


			# Handling of viewing
			PostView($database, $cgi, $param, \%board); 
		}
	}

	# Document footer
	$database->DocumentGetFooter();	

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Handle Alternates
#
# Will handle how alternate boards are displayed.  Essentially, this will simply
# create a redirect header sending it to the alternate board if such a thing
# was specified.

sub HandleAlternate
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list

	my $public;						# Public status
	my $altboard;						# Alternate message board
	

	#####################
	# Program area

	# Verification of acceptance
	if (defined($param->{'ROOM'}))
	{
		# Retreive public status
		($public) = $database->DataGetRealmPublic($param->{'ROOM'});

	
		# Verify existence and nature	
		if ((defined($public)) && ($public ne 'no'))
		{
			# Retreive alternate
			($altboard) = $database->DataGetRealmAltBoard($param->{'ROOM'});		
			

			# Exitence of alternate header
			if (defined($altboard))
			{
				# Retriect header
				print $cgi->redirect($altboard);

				# End script
				exit;
			}
		}
		
		# Private or nonexistent	
		else
		{
			# Dispose of value
			$param->Cleanup('ROOM');
		}
	}

	# Display standard header
	print $cgi->header();
}


#####################
# Realm Selection 
#
# Realms are needed in order to complete operations for the message board.  This
# is for the primary reason that the board grants one topic area for each specific
# realm.  Without a public realm, the board topic will not exist.

sub RealmSelect
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handler
	my $board    = shift;					# Board hash handle 


	my $choice;						# Choice of the matter
	my $count;						# Topic counter
	my $table;						# Table handle
	my $title;						# Title to display

	my $dis;						# Display realms
	my @dis;						# Display selection


	#####################
	# Program area

	# Generate widgets
	# Realm selection
	unless (defined($param->{'ROOM'}))
	{
		# Retreive realm list
		$database->GetListPublicRealms(\@dis);

		# Format line for use
		$dis = $cgi->scrolling_list('ROOM', \@dis, $dis[0], 10);


		# Title assign
		$title = $board->{'TagRealm'};	
	}

	# Topic selection
	elsif (!defined($param->{'TOPIC'}))
	{
		# Retreive topic list
		$database->GetListTopics(\@dis, $param->{'ROOM'});

		# Need for a default topic
		($choice) = $database->DataGet("SELECT COUNT(*) AS 'Topic Posts'
			FROM Board
			WHERE RealmName=?
			  AND BoardTopic='system'",
		 $param->{'ROOM'});

		# Count topics
		$count = @dis;


		# Only show this there are topics
		if ($count > 0)
		{
			# Append default
			unshift(@dis, $board->{'TxtDefault'}) if ($choice > 0);

			# Format line for use
			$dis = $cgi->popup_menu('TOPIC', \@dis, $dis[0], 5);


			# Title assign
			$title = $board->{'TagTopic'};	
		}

		# Assign instead
		else
		{
			# Assign the parameter
			$param->{'TOPIC'} = $board->{'TxtDefault'};
		}
	}


	# Display Realm Prompt
	if (defined($title))
	{
		# Beginning of form
		print $cgi->start_form();

		# Display Table
		$table = new Ethereal::Table($cgi);

		# Top title
		$table->MakeTop();
		$table->MakeSingle($tmpl->PassTitle($title));
		$table->MakeBlank();

		# Widgets and end
		$table->MakeValid($dis, $cgi->submit($board->{'TxtSelect'}));
		$table->MakeBottom();

		# End of form
		print $param->EmbedNormal($cgi, %{$param});
		print $cgi->end_form(), "\n";

		# Return false
		return 0;
	}

	# True if reached
	return 1;
}

#####################
# Post Viewer 
#
# The viewing sub-routine for the message board.  Of course the specific realm is needed
# which is why that this is called after the ROOM is determined. 

sub PostView 
{
	#####################
	# Data members
	my $database = shift;				# Database handle
	my $cgi      = shift;			 	# CGI handle	
	my $param    = shift;				# Parameter list
	my $board    = shift;				# Board hash

	my $data     = $database->{'HANDLE'};		# Database connection handly
	my $date     = new Ethereal::Date();		# Time handler

	my $statement;					# Query handle
	my $res;					# Results hash

	my $dtime    = 86400;				# Measurement of a day
	my $time     = time;				# Time
	my $stime;					# Search time

	my $left;					# Rows left of entry
	my $right;					# Rows for entry

	my $choice;					# Choice of the matter
	my $super;					# Supervisory parameter list
	my $user;					# Username

	my $rows     = 10;				# Complete set of rows
	my $count    = 0;				# Rows retreived
	my $order;					# Order to take on
	my $activity;					# Activity based searching

	my $inline;					# Inline parameter list
	my $url      = $cgi->url();			# Self referencing link

	my %args;					# Basic arguments


	#####################
	# Program area

	# Logged on specific
	if (defined($param->{'USER'}))
	{
		# Determine supervisory
		($choice) = $database->DataGetRoomPrivs($param->{'ROOM'}, $param->{'USER'});

		# Base arguments
		%args = (
			USER  => $param->{'USER'},
			CRYPT => $param->{'CRYPT'},
		 );
	}


	# Generate parameter lines
	# Back button
	my $back = $param->EmbedInline($cgi, %args);

	# Add additional arguments
	$args{'ROOM'}  = $param->{'ROOM'};


	# Topic return
	my $topc = $param->EmbedInline($cgi, %args);

	# Add topic as argument
	$args{'TOPIC'} = $param->{'TOPIC'};

	# Default search
	# Activity based search
	if ((!defined($param->{'ORDER'}))
	 && (!defined($param->{'ACTV'}))
	 && (!defined($param->{'TIME'})))
	{
		# Create to cause default
		$param->{'ACTV'} = 'True';

	}

	# Maintain directional state
	$param->{'ORDER'} = (defined($param->{'ORDER'})) ? $param->{'ORDER'} : 'ASC';


	# Fucntional
	my $post = $param->EmbedInline($cgi, %args, 'POST', 'True');

	# Activity based
	my $actv = $param->EmbedInline($cgi, %args, 'ACTV', 'True');

	# Order
	my $desc = $param->EmbedInline($cgi, %args, 'ORDER', 'DESC');
	my $asc  = $param->EmbedInline($cgi, %args, 'ORDER', 'ASC');


	# Append order
	$args{'ORDER'} =  $param->{'ORDER'};

	# Time based
	my $day  = $param->EmbedInline($cgi, %args, 'TIME', '1');
	my $week = $param->EmbedInline($cgi, %args, 'TIME', '7');
	my $mnth = $param->EmbedInline($cgi, %args, 'TIME', '31');
	my $year = $param->EmbedInline($cgi, %args, 'TIME', '365');


	# Additional searching
	$stime    = (defined($param->{'TIME'})) ? ($time - ($param->{'TIME'} * $dtime)) : 0;
	$activity = (defined($param->{'ACTV'})) ? 'BoardUpdate DESC,' : '';


	# Prepare and execute
	$statement = $data->prepare("SELECT BoardIDNumber,
			 PuppetName,
			 BoardSubject,
			 BoardTimestamp,
			 BoardMessage,
			 BoardLevel,
			 LevelZero,
			 LevelOne,
			 LevelTwo,
			 LevelThree,
			 LevelFour,
			 LevelFive
			FROM  Board 
			WHERE RealmName=?
			  AND BoardTopic=?
			  AND BoardTimestamp > $stime
			ORDER BY $activity
			 LevelZero $param->{ORDER}, 
			 LevelOne,
			 LevelTwo,
			 LevelThree,
			 LevelFour,
			 LevelFive"); 
	$statement->execute($param->{'ROOM'}, $param->{'TOPIC'});

	
	# HTML Display
	print "<BR>\n\n<CENTER><TABLE WIDTH=\"95%\" CELLSPACING=\"1\" CELLPADDING=\"4\">\n";

	# Status view
	print "\t<TR BGCOLOR=\"$board->{SetColorHead}\">",
	      "\t\t<TD COLSPAN=\"$rows\"><FONT COLOR=\"$board->{SetColorTextH}\">",
	      "<A HREF=\"$url$post\">$board->{TagPost}</A> - ",
      	      "<A HREF=\"$url$actv\">$board->{TagActv}</A> - ",
      	      "<A HREF=\"$url$asc\">$board->{TagAsc}</A> ",
      	      "<A HREF=\"$url$desc\">$board->{TagDesc}</A> - ",
      	      "<A HREF=\"$url$day\">$board->{TagTimeDay}</A> ",
      	      "<A HREF=\"$url$week\">$board->{TagTimeWeek}</A> ",
      	      "<A HREF=\"$url$mnth\">$board->{TagTimeMonth}</A> ",
      	      "<A HREF=\"$url$year\">$board->{TagTimeYear}</A> - ",
      	      "<A HREF=\"$url$topc\">$board->{TagSlct}</A> - ",
      	      "<A HREF=\"$url$back\">$board->{TagBack}</A></FONT></TD>\n",
	      "\t</TR>\n";

	# Retreive and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Increment counter
		$count++;

		# Biographical links
		# Retreive puppeteer login
		($user)   = $database->DataGetPuppeteerLogin($res->{'PuppetName'});

		# Inline parameter
		($inline) = $param->EmbedInline($cgi, 'CHAR', $res->{'PuppetName'}, 'USER', $user);


		# Determine rows
		$right = $rows - $res->{'BoardLevel'};
		$left  = $rows - $right;

		# Generate inline
		$inline  = $param->EmbedInline($cgi, %{$param}, 'REPLY', $res->{'BoardIDNumber'});

		# Kill button
		if (defined($choice))
		{ $super = $param->EmbedInline($cgi, %{$param}, 'PURGE', $res->{'BoardIDNumber'}); }

		# Header
		print "\t<TR>\n";

		# Handling based on level
		# Childer
		if ($res->{'BoardLevel'} != 0)
		{
			print "\t\t<TD COLSPAN=\"$left\">&nbsp;</TD>\n";
			print "\t\t<TD COLSPAN=\"$right\" BGCOLOR=\"$board->{SetColorHead}\">",
		      	      "<FONT COLOR=\"$board->{SetColorTextH}\">$board->{TagHeadSubject}$res->{BoardSubject}</FON
T><BR>",
			      "<FONT COLOR=\"$board->{SetColorTextH}\">$board->{TagHeadFrom}",
			      "<A HREF=\"$board{TagLnkBio}$inline\" TARGET=\"_new\">$res->{PuppetName}</A></FONT>",
			      " $board->{TagHeadDate} ", $date->GetDate($res->{'BoardTimestamp'}), " ($res->{BoardIDNumber})</FONT><BR>",
			      "</TD>\n";

			# Kill button
			if (defined($super))
			{
				# Administrative login
				print "\t<TR>\n",
				      "\t\t<TD COLSPAN=\"$left\">&nbsp;</TD>\n",
				      "\t\t<TD COLSPAN=\"$right\" BGCOLOR=\"$board->{SetColorHead}\">",
				      "<A HREF=\"$url$super\">$board->{TagKillThread}</A>",
				      "</TD>\n",
				      "\t</TR>\n";
			}
		}

		# Parent
		else
		{
			print "\t\t<TD COLSPAN=\"$rows\" BGCOLOR=\"$board->{SetColorHead}\">",
		      	      "<FONT COLOR=\"$board->{SetColorTextH}\">$board->{TagHeadSubject}$res->{BoardSubject}</FONT><BR>",
			      "<FONT COLOR=\"$board->{SetColorTextH}\">$board->{TagHeadFrom}",
			      "<A HREF=\"$board{TagLnkBio}$inline\" TARGET=\"_new\">$res->{PuppetName}</A></FONT>",
			      " $board->{TagHeadDate} ", $date->GetDate($res->{'BoardTimestamp'}), " ($res->{BoardIDNumber})</FONT><BR>",
			      "</TD>\n";

			# Kill button
			if (defined($super))
			{
				# Administrative login
				print "\t<TR>\n",
				      "\t\t<TD COLSPAN=\"$rows\" BGCOLOR=\"$board->{SetColorHead}\">",
				      "<A HREF=\"$url$super\">$board->{TagKillThread}</A>",
				      "</TD>\n",
				      "\t</TR>\n";
			}
		}

		print "\t</TR>\n";


		# Message
		print "\t<TR>\n";

		# Handling based on level
		# Children of threads
		if ($res->{'BoardLevel'} != 0)
		{
			print "\t\t<TD COLSPAN=\"$left\">&nbsp;</TD>\n";
			print "\t\t<TD COLSPAN=\"$right\" BGCOLOR=\"$board->{SetColorBody}\">",
			      "<FONT COLOR=\"$board->{SetColorText}\">$res->{BoardMessage}</FONT>",
			      "<BR><A HREF=\"$url$inline\">$board->{TagReply}</A>",
			      "</TD>\n";
		}

		# Threads parent
		else
		{
			print "\t\t<TD COLSPAN=\"$rows\" BGCOLOR=\"$board->{SetColorBody}\">",
			      "<FONT COLOR=\"$board->{SetColorText}\">$res->{BoardMessage}</FONT>",
			      "<BR><A HREF=\"$url$inline\">$board->{TagReply}</A>",
			      "</TD>\n";
		}
	
		print "\t</TR>\n";
	}

	# Status view
	print "\t<TR BGCOLOR=\"$board->{SetColorHead}\">",
	      "\t\t<TD COLSPAN=\"$rows\"><FONT COLOR=\"$board->{SetColorTextH}\">$board->{TagCount} $count<BR>",
	      "<A HREF=\"$url$post\">$board->{TagPost}</A> - ",
      	      "<A HREF=\"$url$topc\">$board->{TagSlct}</A> - ",
      	      "<A HREF=\"$url$back\">$board->{TagBack}</A></FONT></TD>\n",
	      "\t</TR>\n";


	# Spacer row
	print "\t<TR>\n";
	for (my $i; $i < $rows; $i++) { print "\t\t<TD>&nbsp;</TD>\n"; } 

	# End of table
	print "\t</TR>\n</TABLE></CENTER>\n";


	# Close query handle
	$statement->finish();
}


#####################
# Post Composer  
#
# As the name would imply, this subroutine allows for the addition of posts to the 
# existing set of posts.  Provided that there are posts to reply to, one can
# reply under the parent post.

sub PostComposer
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $login    = shift;					# Login handler
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handler
	my $board    = shift;					# Board hash handle 

	my $acc;						# Realm access
	my $ban;						# User banned status
	my $def;						# Default page
	my $subj;						# Subject of message
	my $table;						# Table handle

	my $puppets;						# Prepared output
	my @puppets;						# List of puppets


	#####################
	# Program area
	
	# User logged on
	if ($login->GetVerificationNormal())
	{
		# Determine ban
		($acc) = $database->DataGetRealmAccess($param->{'ROOM'});
		($ban) = $database->DataGetBannedStatus($param->{'USER'}, $param->{'ROOM'});

		# Default
		($def) = $database->DataGetDefault($param->{'USER'});


		# Define instance
		$table = new Ethereal::Table($cgi);

		# Allow if not banned
		unless ((defined($ban)) && ($acc eq 'relaxed'))
		{
			# Gather information
 			if (defined($param->{'REPLY'}))
			{
				# Prepare and execute
			 	($subj) = $database->DataGet("SELECT BoardSubject 
					FROM Board 
					WHERE BoardIDNumber=?", $param->{'REPLY'});	

				# Adapt subject
				if ($subj !~ /^\(RE\)/)
				{ $subj =~ s/(.+)/(RE) $1/; }

				# Assign ID for next pass
				$param->{'PID'} = $param->{'REPLY'};
			}

			# Cleanup
			$param->Cleanup('REPLY', 'POST');


			# Puppet list 
			# Retreive puppet list
			$database->GetListPuppetYours(\@puppets, $param->{'USER'});

			# Determine puppets
			$puppets = $cgi->popup_menu('PFROM', \@puppets, $def);


			# Display area
			# Display title
			$tmpl->ShowMain($board->{'TagTitlePost'});

			# Start of form
			print $cgi->start_form();
			print $param->EmbedNormal($cgi, %{$param}, 'WRITE', 'True'), "\n";

			# Table top
			$table->MakeTop();

			# Header
			$table->MakeSingle($tmpl->PassTitle($board->{'TagPostHead'}));
			$table->MakeValid(
			 $tmpl->PassSub($board->{'TagPostFrom'}),
			 $puppets);
			$table->MakeValid(
			 $tmpl->PassSub($board->{'TagPostSubject'}),
			 $cgi->textfield('PSUBJ', $subj,25,250));
	
			# Body
			$table->MakeBlank();
			$table->MakeSingle($tmpl->PassTitle($board->{'TagPostMessage'}));
			$table->MakeSingle($cgi->textarea(-name=>'PTEXT',-rows=>10,-columns=>60,-wrap=>'SOFT'));


			# Submit and clear
			$table->MakeBlank();
			$table->MakeValid($cgi->submit($board->{'TxtPost'}), $cgi->reset($board->{'TxtReset'}));
			$table->MakeBottom();

			# End form
			print $cgi->end_form(), "\n";
		}


		# Banner users
		else
		{
			# Simple table
			# Display the news
			$table->MakeTop();
			$table->MakeSingle($board->{'MsgBanned'});
			$table->MakeBottom();
		}
	}
}


#####################
# Post Purge
#
# Handles the purging or killing of post and any children associated with it.   As one can
# see this is useful for giving the message board a more youthful look and cleaning out
# unwanted material.

sub PostPurge
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $login    = shift;					# Login handle
	my $cgi	     = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $super;						# Supervisory access

	#####################
	# Program Area

	# Only if login credentials are valid
	if ($login->GetVerificationNormal())
	{
		# Audit handle
		my $audit = new Ethereal::Audit($database, $cgi, $param);

		# Superisory access
		($super)  = $database->DataGetRoomPrivs($param->{'ROOM'}, $param->{'USER'});

		# Ensure they are supervisors
		if (defined($super)) 
		{
			# Pruge posts
			$data->do("DELETE FROM Board
				WHERE RealmName=? 
				  AND BoardTopic=?
				  AND (LevelZero=? 
				   OR LevelOne=?
				   OR LevelTwo=?
				   OR LevelThree=?
				   OR LevelFive=?)", {},

				 $param->{'ROOM'},
				 $param->{'TOPIC'},
				 $param->{'PURGE'},	
				 $param->{'PURGE'},	
				 $param->{'PURGE'},	
				 $param->{'PURGE'},	
				 $param->{'PURGE'});
		}

		# Auditing table
		$audit->BoardRem($param->{'PURGE'});

		# Cleanup
		$param->Cleanup('PURGE');
	}

	# Kill switch
	else
	{
		# Do not collect 100$
		$param->{'PASSGO'} = 'No';
	}

}


#####################
# Post Writer  
#
# Handles the eventual cleanup and posting of the post to the threads and tree.  This includes
# language filtering and removal of HTML to keep this clean.
	
sub PostWrite	 						# Database, CGI, Parameter list
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $login    = shift;					# Login handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $board    = shift;					# Board hash handle
	my $system   = shift;					# System board

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $date     = new Ethereal::Date();			# Date handler
	my $filter   = new Ethereal::Filter();			# Filter handler
	my $send;						# Mail sender

	my $statement;						# Query statement

	my $time;						# Date of handling
	my $email;						# Email address
	my $name;						# Full name
	my $notice;						# Notice
	my $post;						# Pre modified post

	my $puppet;						# Puppet name
	my $puppeteer;						# Puppeteer name

	my $level;						# Level
	my $res;						# Results

	my $id  = 'LAST_INSERT_ID()';				# ID insertion function
	my $url = $cgi->url();					# Location to self

	my @levels;						# Levels and depth


	#####################
	# Program area

	# Wrapper
	$Text::Wrap::columns = 72;


	# Link with hash
	$database->GetHashSystem(\%system);

	# Ensure authenthication
	if ($login->GetVerificationNormal())
	{
		# Information handling
		# Retreive post information
		if (defined($param->{'PID'}))
		{
			# Prepare and execute
			$statement = $data->prepare("SELECT PuppetName,
					 BoardLevel,
					 LevelZero,
					 LevelOne,
					 LevelTwo,
					 LevelThree,
					 LevelFour,
					 LevelFive
					FROM  Board
					WHERE BoardIDNumber=?");
			$statement->execute($param->{'PID'});

			# Retreive and adapt
			$res = $statement->fetchrow_hashref();

			# Determine levels
			$levels[0] = $res->{'LevelZero'};
			$levels[1] = ($res->{'LevelOne'}   != $res->{'LevelZero'})  ? $res->{'LevelOne'}   : $id;
			$levels[2] = ($res->{'LevelTwo'}   != $res->{'LevelOne'})   ? $res->{'LevelTwo'}   : $id;
			$levels[3] = ($res->{'LevelThree'} != $res->{'LevelTwo'})   ? $res->{'LevelThree'} : $id;
			$levels[4] = ($res->{'LevelFour'}  != $res->{'LevelThree'}) ? $res->{'LevelFour'}  : $id;
			$levels[5] = ($res->{'LevelFive'}  != $res->{'LevelFour'})  ? $res->{'LevelFive'}  : $id;

			# Determin response level
			$level = $res->{'BoardLevel'} + 1;
			$level = ($level > 5) ? 5 : $level; 

			# Determin puppet name
			$puppet = $res->{'PuppetName'};


			# Finish query
			$statement->finish();
		}
	
		# Parent (root level) posts
		else
		{
			# Assign levels and level for root
			@levels = ($id, $id, $id, $id, $id, $id); 
			$level = 0; 
		}

		# Sanity check
		$param->{'PTEXT'} = $filter->MakeSane($param->{'PTEXT'}, $board->{'SetTruncate'});


		# Mold subject and body
		# Strip HTML
		$param->{'PSUBJ'} =~ s/<[^>]*>//gs;
		$param->{'PTEXT'} =~ s/<[^>]*>//gs;

		# Buffer for Emails
		$post =  fill('    ', '    ', $param->{'PTEXT'});

		# Replace carriage returns
		$param->{'PTEXT'} =~ s/\n/<BR>/gs;

		# Create hyperlinks
		$param->{'PTEXT'} =~ s/(http:\/\/.*?)(\s|$)/<A HREF="$1" TARGET="_blank">$board->{TagHyperlink}<\/A> /gm;
		$param->{'PTEXT'} =~ s/([\w\.\-]*?\@[\w\.\-]*?\..*?)(\s|$)/<A HREF="mailto:$1">$board->{TagEmail}<\/A> /gm;

		# Database insertion
		$data->do("INSERT INTO Board 
			(BoardIDNumber,
			 PuppetName,
			 RealmName,
			 BoardTopic,
			 BoardSubject,
			 BoardTimestamp,
			 BoardMessage,
			 BoardUpdate,
			 BoardLevel,
			 LevelZero,
			 LevelOne,
			 LevelTwo,
			 LevelThree,
			 LevelFour,
			 LevelFive)
			VALUES
			(NULL,?,?,?,?,?,?,?,?,0,0,0,0,0,0)", {},

		 $param->{'PFROM'},
		 $param->{'ROOM'},
		 $param->{'TOPIC'},
		 $param->{'PSUBJ'},
		 time,
		 $param->{'PTEXT'},
		 time,
		 $level);

		# MySQL kludge
		$data->do("UPDATE Board
			SET LevelZero=$levels[0],
			    LevelOne=$levels[1],
			    LevelTwo=$levels[2],
			    LevelThree=$levels[3],
			    LevelFour=$levels[4],
			    LevelFive=$levels[5]
			WHERE BoardIDNumber=LAST_INSERT_ID()", {});

		# Bump up process
		$data->do("UPDATE Board
			SET BoardUpdate=?
			WHERE LevelZero=?", {}, 

		 time,
		 $levels[0]);

		# Generate date
		$time = $date->GetDate(time);	

		# Administrative message
		# Determine need
		($email)  = $database->DataGetRealmBoard($param->{'ROOM'});


		# Generate
		if (defined($email))
		{
			# Pull
			($notice) = $database->DocumentGetBoardAdmin();


			# Send Mail
			# Handle initiation
			$send = tie(*MAIL, 'Ethereal::Mail', $database);

			# Initial setup
			$send->SetSubject("$system->{SetBoardPrefix} $param->{PSUBJ}");
			$send->SetSearch(
				MDATE => $time,
				MFROM => $param->{'PFROM'},
				MLINK => $url,
				MMAIL => "$system->{SetInfoContactName}\n$system->{SetInfoContactAddress}",
				MMESG => $post,
				MROOM => $param->{'ROOM'},
				MSUBJ => $param->{'PSUBJ'},
				MTOPC => $param->{'TOPIC'}
			 );

			# Recipients
			$send->AddTo("$name <$email>");

			# Add message
			print MAIL $notice;

			# Close and send
			close(MAIL);
		}

		# Email notification
		if (defined($puppet))
		{
			# Retreive login
			($puppeteer) = $database->DataGetPuppeteerLogin($puppet);


			# Complete if defined
			if (defined($puppeteer))
			{
				# Retreive name and Email
				($name)   = $database->DataGetPuppeteerName($puppeteer);
				($email)  = $database->DataGetPuppeteerEmail($puppeteer);
				($notice) = $database->DocumentGetBoardNotice();


				if (defined($notice))
				{
					# Send Email
					# Iniate instance
					$send = tie(*MAIL, 'Ethereal::Mail', $database);

					# Initial setup
					$send->SetSubject("You have a reply on $param->{ROOM} message board from: $param->{'PFROM'}");
					$send->SetSearch(
						MDATE => $time,
						MFROM => $param->{'PFROM'},
						MLINK => $url,
						MMAIL => "$system->{SetInfoContactName}\n$system->{SetInfoContactAddress}",
						MMESG => $post,
						MROOM => $param->{'ROOM'},
						MSUBJ => $param->{'PSUBJ'},
						MTOPC => $param->{'TOPIC'}
					 );

					# Recipients
					$send->AddTo("$name <$email>");

					# Message
					print MAIL $notice;

					# Close and send
					close(MAIL);
				}
			}
		}

		# Cleanup
		$param->Cleanup(
			'PID',
			'PFROM',
			'PSUBJ',
			'PTEXT',
			'WRITE'
		 );
	}

	# Kill switch
	else
	{
		# Do not collect 100$
		$param->{'PASSGO'} = 'No';
	}

}
